@@ -1,14 +1,18 @@
 #!/usr/bin/env python3
 
 # internal:
-from fdroid_mirror_monitor import collect, history, report, website
-from fdroid_mirror_monitor.utils import parallelize, get_logger, geoip, own_ip
+from fdroid_mirror_monitor import repo, mirror, history, website
+from fdroid_mirror_monitor.utils import get_logger, geoip, get_own_ip, get_mirrors_readme
 # stdlibs
 from datetime import datetime, timezone
 from importlib.metadata import metadata
 from os import makedirs, path
+from queue import Queue
 import argparse
 import json
+import threading
+import traceback
+import time
 
 
 def main():
@@ -27,15 +31,17 @@ def main():
     parser.add_argument('-d', '--directory',
                         help='output directory',
                         default='public')
-    parser.add_argument('-m', '--mirrors',
-                        help='Comma seperated list of mirrors to check')
-    parser.add_argument('-r', '--repos',
-                        help='Extract repos from izzy, wiki',
+    parser.add_argument('--wiki',
+                        help='Include repos from https://forum.f-droid.org/t/known-repositories/721',
                         action='store_true')
     parser.add_argument('-t', '--threads',
                         help='Number of parallel threads when checking mirrors',
                         choices=range(1, 10), type=int,
                         default=5)
+    parser.add_argument('--timeout',
+                        help='Timeout for requests (seconds)',
+                        type=int,
+                        default=60)
     parser.add_argument('--trim',
                         help='Trim history entries to the last x hours',
                         type=int,
@@ -50,6 +56,7 @@ def main():
 
     # init logger
     log = get_logger(__name__, args.verbosity)
+    log.debug('logging is set to debug')
 
     if args.html:
         with open(output_dir + '/status.json', 'r') as fp:
@@ -57,37 +64,91 @@ def main():
             website.create_html(report=status, dir=output_dir)
             exit(0)
 
-    if args.mirrors:
-        mirrors = args.mirrors.split(',')
-    else:
-        log.info('Collecting mirrors')
-        mirrors = collect.mirrors()
+    # if args.mirrors:
+    #     mirrors = args.mirrors.split(',')
+    # else:
+    #     log.info('Collecting mirrors')
+    #     mirrors = collect.mirrors()
 
-    # TODO: create report with mirror URL instead of repo URL,
-    # check for archive in create_report() instead of README
-    repo_urls = []
-    for mirror_url in mirrors:
-        repo_urls.append(mirror_url + '/repo')
+    # if args.repos:
+    #     log.info('Collecting repos')
+    #     mirrors += collect.repos()
 
-    if args.repos:
-        log.info('Collecting repos')
-        repo_urls += collect.repos()
+    # log.debug('mirrors: %s', mirrors)
 
+    # official repos
+    repo_mirrors, archive_mirrors = get_mirrors_readme()
+
+    official_fp = '43238D512C1E5EB2D6569F4A3AFBF5523418B82E0A3ED1552770ABB9A9C9CCAB'
+    url = 'https://f-droid.org/repo'
+    official_repo = repo.Repo(url, fingerprint=official_fp)
+    official_repo.set_additional_mirrors(repo_mirrors)
+    # success = official_repo.get_all()
+    # if success is False:
+    #     exit('Error: https://f-droid.org/repo')
+
+    # url_archive = 'https://f-droid.org/archive'
+    # official_repo_archive = repo.Repo(url_archive, fingerprint=official_fp)
+    # official_repo_archive.set_additional_mirrors(archive_mirrors)
+    # success = official_repo_archive.get_all()
+    # if success is False:
+    #     exit('Error: https://f-droid.org/archive')
+
+    #################################
+    # run all mirror tests as threads
+    mirrors = mirror.Mirror.instances
     timestamp = int(datetime.now(timezone.utc).timestamp())
+
+    def worker(q, exceptions):
+        while not q.empty():
+            time.sleep(1)
+            mirror_obj = q.get()
+            threading.currentThread().name = mirror_obj.name
+            try:
+                mirror_obj.run_all()
+                q.task_done()
+            except Exception as e:
+                log.fatal(e)
+                log.error(traceback.format_exc())
+                exceptions[mirror_obj.name] = e
+                q.task_done()
+
+    # put all mirrors in queue
+    jobs = Queue()
+    for mirror_obj in mirrors:
+        jobs.put(mirror_obj)
+
+    exceptions = {}
+    # start threads
+    for i in range(args.threads):
+        t = threading.Thread(target=worker, args=(jobs, exceptions,), daemon=True)
+        t.start()
+
+    # wait for threads to finish
+    jobs.join()
+
+    for t in exceptions:
+        log.fatal('Exception in thread %s: %s' % (t, exceptions[t]))
+    if len(exceptions) > 0:
+        exit(1)
+
+    ###############
+    # create status
     mirror_reports = []
-    mirror_report_dict = parallelize(repo_urls, report.create, args.threads)
-    for mirror in mirror_report_dict:
-        mirror_reports.append(mirror_report_dict[mirror])
+    for mirror_obj in mirrors:
+        mirror_reports.append(mirror_obj.get_data())
 
     status = dict()
     status['version'] = metadata(__package__)['Version']
     status['last_check'] = timestamp
-    status['check_ip'] = own_ip()
+    status['check_ip'] = get_own_ip()
     status['check_country'], status['check_country_code'] = geoip(status['check_ip'])
     status['mirrors'] = mirror_reports
 
     makedirs(output_dir, exist_ok=True)
 
+    ##################################################
+    # create history, add historical context to status
     if args.no_history is False:
         log.info('Adding report to history')
         try:
@@ -109,10 +170,13 @@ def main():
 
         h.save(path.join(args.directory, 'history.json'))
         status = h.average_status(args.trim)
+    ##################################################
 
+    # save status.json
     with open(path.join(output_dir, 'status.json'), 'w') as fp:
         json.dump(status, fp)
 
+    # create HTMLs
     if args.no_html is False:
         log.info('Creating HTMLs')
         website.create_html(report=status, dir=output_dir)
